/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */
var tooltip = ripple('ui/plugins/tooltip'),
    settings = ripple('ui/plugins/settings-dialog'),
    bus = ripple('bus');

function handleBuild() {
    var node = $(this),
        action = node.attr('id').split("-")[2];

    if (node.hasClass("not-ready")) {
        return;
    }

    if (!settings.can(action)) {
        settings.show(action);
    }
    else {
        settings.perform(action);
    }
}

function areServicesRunning(success, fail) {
    bus.ajax(
        "GET",
        "http://127.0.0.1:9910/ripple/about",
        null,
        function () {
            success();
        },
        function () {
            fail();
        }
    );
}

function manageServices(action, callBack) {
    bus.send("services", action, function () {
        if (typeof callBack === "function") {
            callBack();
        }
    });
}

module.exports = {
    panel: {
        domId: "build-container",
        collapsed: true,
        pane: "right"
    },

    initialize: function () {
        $("#options-menu-build").click(handleBuild);
        $("#options-menu-sign").click(handleBuild);
        $("#options-menu-launch").click(handleBuild);

        $('#options-menu-start-service').click(function () {
            manageServices("start", function () {
                $('#options-menu-services-stop').show();
                $('#options-menu-services-start').hide();
            });
        });

        $('#options-menu-stop-service').click(function () {
            manageServices("stop", function () {
                $('#options-menu-services-stop').hide();
                $('#options-menu-services-start').show();
            });
        });

        $("#options-menu-settings").click(function () {
            settings.show();
        });

        if (settings.value("remoteInspector")) {
            $("#options-menu-build-warning").show();
            tooltip.create("#options-menu-build-warning", "Remote Web Inspector should be disabled when packaging for App World release");
        }

        areServicesRunning(
            function () {
                $('#options-menu-services-stop').show();
                $('#options-menu-services-start').hide();
            },
            function () {
                $('#options-menu-services-stop').hide();
                $('#options-menu-services-start').show();
            }
        );
    }
};
